/* (C) Marcin Kwadrans <quar@vitea.pl> */

#include "include/support.h"
#include "include/board.h"

static void clear_mark (LWPiece **marked_piece)
{
	*marked_piece = NULL;
}

/*!	\brief Inicjalizacja planszy

	Inicjalizuje obiekt, metoda prywatna używana wewnętrznie przez konstruktor

	\param type Typ planszy
*/
void LWBoard::init (LWType a_type)
{
	widget = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (widget);
	
	type = a_type;

	if (type == LW_TYPE_PROGRAM) {
		dummy_row = new LWRow(this);

		gtk_box_pack_end (GTK_BOX (widget), dummy_row->getWidget(), TRUE, TRUE, 0);
	}
}

/*! 
 	\brief Konstruktor klasy
	
	Konstruktor klasy, na podstawie typu. W zależności od zastosowania
	planszy będą używane różne typy.
	
	\param type Typ tworzonej planszy
*/
LWBoard::LWBoard (LWType a_type):
list_row(NULL), dummy_row(NULL), marked_piece(NULL), boardset(NULL)
				
{
	init (a_type);
}

/*! 
	\brief Konstruktor klasy
 
	Konstruktor klasy, na podstawie zestawu plansz do którego będzie należała
	plasza. Typ tworzonej planszy będzie ustalony na taki sam jakiego używa
	zestaw. 
*/
LWBoard::LWBoard (LWBoardSet *a_boardset): 
list_row(NULL), dummy_row(NULL), marked_piece(NULL)
{
	init (a_boardset->getType());
	
	boardset = a_boardset;
}

/*! \brief Destruktor klasy

	Destruktor klasy, podczas destrukcji planszy usunięte również wszystkie 
	należące do niej wiersze.
*/
LWBoard::~LWBoard ()
{
	clear();

	if (dummy_row != NULL)
		delete dummy_row;
	
	gtk_widget_destroy (widget);
}

/*!	\brief Konstruktor kopiujący
 
	Konstruje planszę oraz kopiuje wszystkie należące do niej 
	wiersze. Nowa plansza przyjmuje typ według argumentu typ

	\param board Kopiowana plansza
	\param type Typ tworzonej planszy 
*/
LWBoard::LWBoard (const LWBoard *board, LWType a_type):
list_row(NULL), dummy_row(NULL), marked_piece(NULL), boardset(NULL)

{
	init (a_type);
	
	for (GSList *l=board->list_row; l != NULL; l = l->next) {
		LWRow *row = (LWRow *) l->data;
		LWRow *newrow = new LWRow (row, this);
		addRow (newrow);
	}
}

/*!	\brief Rekonstrukcja elementów planszy na podstawie opisu XML

	Rekonstrukcja planszy na podstawie opisu XML zawartego w węźle 
	drzewa XML. Poprzednia zawartość planszy zostaje usunięta.

	\param node Węzeł drzewa XML na postawie którego dokonana zostanie
				rekonstrukcja
*/
void LWBoard::restoreFromXML (xmlNode *node)
{
	g_return_if_fail (!xmlStrcasecmp (node->name, (xmlChar *) "Board"));

	gchar *title = (gchar *) xmlGetProp (node, (xmlChar *) "title");	
	
	clear();
	
	if (boardset != NULL && title != NULL)
		boardset->changeBoardTitle (this, title);
	
	for (xmlNode *n=node->children; n != NULL; n=n->next) {
		LWRow *row = new LWRow (this);
		row->restoreFromXML (n);
		addRow (row);
	}
}

/*! \brief Zapis elementów planszy w pastaci opisu XML
 
	Zapis planszy do postaci węzła drewa XML, utworzony węzeł zostanie dodany
	jako struktury rodzica.

	Nie jest dokonywane sprawdzenie nazwy węzła rodzica, najczęściej
	będzie to korzeń.

	\param node Węzeł drzewa XML na postawie którego dokonanie zostanie
				rekonstrukcja
*/
void LWBoard::storeToXML (xmlNode *parent_node)
{
	xmlNode *node = xmlNewChild(parent_node, NULL, (xmlChar *) "Board", NULL);

	for (GSList *l = list_row; l != NULL; l = l->next) {
		LWRow *row = (LWRow *) l->data;
		row->storeToXML (node);
	}
	
}

/*! \brief Dodanie wiersza do planszy
 
	Dodanie wiersza do planszy, dadany wiersz będzie znajdował się na dole 
	planszy. 

	\param row Dodawany wiersz
*/
void LWBoard::addRow (LWRow *row)
{
	g_return_if_fail (row != NULL);
	g_return_if_fail (row->board == this);
	
	gtk_box_pack_start (GTK_BOX (widget), row->getWidget(), FALSE, FALSE, 0);
	list_row = g_slist_append (list_row, (gpointer) row);
}

/*! \brief Usuwanie wiersza z planszy
 
	Usuwanie wiersza z planszy, usuwany wiersz musi się zawierać w danej instancji
	klasy, w przeciwnym wypadku nie zostanie usunięty.

	\param row Usuwany wiersz
*/
void LWBoard::removeRow (LWRow *row)
{
	g_return_if_fail (row != NULL);
	g_return_if_fail (row->board == this);
	
	list_row = g_slist_remove (list_row, (gpointer) row);
	delete row;
}

/*! \brief Pozycja wiersza w planszy

	Zwraca pozycję weiersza w strukturze planszy.

	\param row Pozycja wiersza na planszy liczona od 0. Jeśli wiersz nie należy
	do planszy zwrócone zostanie -1.
*/
gint LWBoard::getRowIndex (LWRow *row)
{
	g_return_val_if_fail (row != NULL, -1);
	g_return_val_if_fail (row->board == this, -1);
	
	return g_slist_index (list_row, (gpointer) row);
}

/*! \brief Wiersz na danej pozycji

	Zwraca wiersz na planszy który znajduje się na pozycji przekazanej w 
	argumencie. Pozycja na planszy liczona jest od 0.

	\param n Pozycja wiersza, na której się znajduje poszukiwany wiersz.
	\return Zwrócony wiersz, lub NULL w przypadku, gdy nie ma wiersza
	na poszukiwanej pozycji.
*/
LWRow *LWBoard::getRowNth (guint n)
{
	return (LWRow *) g_slist_nth_data (list_row, n);
}

/*! \brief Lista klocków na planszy

	Zwraca listę wszystkich klocków na planszy. 

	\return Zwrócona kopia listy klocków. Listę należy zwolnić
	przez g_slist_free.
*/
GSList *LWBoard::getPieceList ()
{
	GSList *list=NULL;
	
	for (GSList *l = list_row; l != NULL; l = l->next)
		list = g_slist_concat (list, ((LWRow *) l->data)->getPieceList());
	
	return list;
}

/*! \brief Określenie rozmiaru planszy

	Określa ilość klocków w pionie i poziomie
	\param width Szerokość planszy
	\param height Wysokość planszy
*/
void LWBoard::setSize (guint width, guint height)
{
	GSList *l;

	/* Zmniejsz liczbe wierszy jeśli potrzeba */
	while (NULL != (l = g_slist_nth (list_row, height))) {
		LWRow *row = (LWRow *) l->data;
		g_slist_delete_link (list_row, l);
		delete row;
	}
	
	/* Zwiększ liczbę wierszy jeśli potrzeba */
	for (guint i = g_slist_length (list_row); i < height; i++)
		addRow (new LWRow (this));
	
	/* Ustaw poprawną ilość elementów w wierszach */
	for (l = list_row; l != NULL; l = l->next) {
		LWRow *row = (LWRow *) l->data;
		row->setSize (width);
	}
}

/*! \brief Czyszczenie zawarości planszy

	Usuwa wszystkie wiersze na planszy.
	W przypadku gdy typem planszy jest LW_TYPE_PROGRAM
	pozostaje tylko dodatkowy wiersz u dołu planszy przeznaczony
	do upuszczania nowych klocków przy pomocy przeciągnij i upuść.
	Metoda nie przyjmuje argumentów i nie zwraca wyniku
*/
void LWBoard::clear ()
{
	for (GSList *l=list_row; l != NULL; l = l->next) {
		LWRow *row = (LWRow *) l->data;
		delete row;		
	}

	g_slist_free (list_row);
	list_row = NULL;
}

/*! \brief Wyróżnianie klocka

	Wyróżnia klocek, który leży w wierszu, który leży na planszy.
	Wyróżninie klocków jest użyteczne przy syganalizowaniu komunikatów,
	komunikat wyróznia klocek, którego komunikat dotyczy.

	\param piece Klocek do wyróżnienia
*/
void LWBoard::markPiece (LWPiece *piece)
{
const GdkColor markcolor = {0, 0xe000, 0x4000, 0x0000};
	
	g_return_if_fail (piece != NULL);
	g_return_if_fail (piece->row != NULL);
	g_return_if_fail (piece->row->board == this);
	
	if (marked_piece != NULL)
		unmarkPiece();
		
	gtk_widget_modify_bg (piece->getWidget(), GTK_STATE_NORMAL, &markcolor);
	
	g_object_set_data_full ( G_OBJECT (piece->getWidget()), "mark", 
								(gpointer) &marked_piece, (GDestroyNotify) clear_mark);
	
	marked_piece = piece;
}

/*! \brief Usuwanie wyróżnienia klocka

	Usuwa wyróznionienie kloceka, wyróznionego przez metodę markPiece.
	Jeśli nie było zaznaczonego klocka, metoda nic nie robi.
	Metoda nie przyjmuje parametrów.
*/
void LWBoard::unmarkPiece ()
{
	if (marked_piece == NULL) return;
		
	gtk_widget_modify_bg (marked_piece->getWidget(), GTK_STATE_NORMAL, NULL);
	marked_piece = NULL;
}

/*! \brief Pobieranie kontrolki
 
	Pobiera kontrolkę (ang. widget) zawierająca planszę

	\return Zwrócona kontrolka
*/
GtkWidget *LWBoard::getWidget ()
{
	return widget;
}

/*! \brief Pobieranie typu planszy
 
	Pobiera typ planszy

	\return Zwracany typ
*/
LWType LWBoard::getType ()
{
	return type;
}

/*! \brief Sprawdzanie czy element jest dodatkowy

	Sprawdza czy element jest dodatkowym elemntem, 
	Metoda przeznaczona wyłącznie do obsługi mechanizmu 
	Przeciągnij i Upuść.

	\param row Sprawdzany element	
	\return Rezultat testu. TRUE jeśli wiersz jest dodatkowy,
	w przeciwnym wypdaku FALSE
*/
gboolean LWBoard::isRowDummy (LWRow *row)
{
	return (row == dummy_row) ? TRUE : FALSE;
}
