/* (C) Marcin Kwadrans <quar@vitea.pl> */

#include "include/support.h"
#include "include/boardset.h"

/*! \brief Tłumaczenie nazw zakładek

	Tłumaczy nazwę zakładki zawartej w *title.
	Funkcja zapisuje w title wskaźnik do przetłumaczonego
	komunikatu lub nie modyfikuje wskaźnika jeśli komunikat
	nie może być stłumaczony 
	\param title Wskaźnik do nazwy zakładki do przetłumaczenia
*/
static void try_translate_title (gchar **title)
{
gchar *newtitle = NULL;
	
	if (!g_strcasecmp (*title, "Icons"))
		newtitle = g_strdup (_("Icons"));
	else
	if (!g_strcasecmp (*title, "House"))
		newtitle = g_strdup (_("House"));
	else
	if (!g_strcasecmp (*title, "Digits"))
		newtitle = g_strdup (_("Digits"));
	else
	if (!g_strcasecmp (*title, "Alphabet"))
		newtitle = g_strdup (_("Alphabet"));
	else
	if (!g_strcasecmp (*title, "Commands"))
		newtitle = g_strdup (_("Commands"));
	else
	if (!g_strcasecmp (*title, "Wizard"))
		newtitle = g_strdup (_("Wizard"));
	else
	if (!g_strcasecmp (*title, "Variables"))
		newtitle = g_strdup (_("Variables"));
	else
	if (!g_strcasecmp (*title, "Math"))
		newtitle = g_strdup (_("Math"));
	else
	if (!g_strcasecmp (*title, "Conditions and loops"))
		newtitle = g_strdup (_("Conditions and loops"));
	
	
	if (newtitle != NULL) {
		g_free (*title);
		*title = newtitle;
	}
}

/*! \brief Konstruktor zbioru plansz

	Tworzy zbiór plansz typu 
	\param a_type Typ plansz przechowywanych w zbiorze
*/
LWBoardSet::LWBoardSet  (LWType a_type): list_board(NULL)
{
	type = a_type;
	
	widget = gtk_notebook_new ();
	gtk_widget_show (widget);
}

/*! \brief Destruktor

	Destruktor zbioru plansz. Rekurencyjnie niszczy również wszystkie
	plansze zawarte w zbiorze
*/
LWBoardSet::~LWBoardSet ()
{
	clear();
	gtk_widget_destroy (widget);
}

/*! \brief Odtwarza zawartość na podstawie drzewa XML

	Odtworzenie zawartości plansz na podstawie drzewa XML
	\param node Węzeł drzewa zawierający opis zbioru
*/
void LWBoardSet::restoreFromXML (xmlNode *node)
{
	g_return_if_fail (!xmlStrcasecmp (node->name, (xmlChar *) "BoardSet"));
	
	clear();
	
	for (xmlNode *n=node->children; n != NULL; n=n->next) {
		LWBoard *board = new LWBoard (this);
		gchar *title = (gchar *) xmlGetProp (node, (xmlChar *) "title");
		try_translate_title (&title);
		addBoard (board, title);
		g_free (title);
		board->restoreFromXML (n);
	}
}

/*! \brief Dadanie planszy do zbioru

	Dodaje planszę do zbioru nadając jej etykietę
	\param board Plansza
	\param title Etykieta planszy
*/
void LWBoardSet::addBoard (LWBoard *board, gchar *title)
{
	g_return_if_fail (board != NULL);
	g_return_if_fail (title != NULL);
	
	GtkWidget *w = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w),
				      GTK_POLICY_AUTOMATIC,
				      GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_add_with_viewport (
			GTK_SCROLLED_WINDOW (w), board->getWidget());
	gtk_widget_show (w);
	
	gtk_notebook_append_page (GTK_NOTEBOOK (widget), w,
									gtk_label_new (title));
	
	list_board = g_slist_append (list_board, board);
}

/*! \brief Zmiana etykiety

	Zmienia etykietę planszy dodanej do zbioru
	\param borad Plansza
	\param title Etykieta planszy
*/
void LWBoardSet::changeBoardTitle (LWBoard *board, gchar *title)
{
	g_return_if_fail (board != NULL);
	g_return_if_fail (title != NULL);
	
	GtkWidget *w = gtk_widget_get_ancestor (board->getWidget(),
		GTK_TYPE_SCROLLED_WINDOW);
	
	g_return_if_fail (w != NULL);
	
	try_translate_title (&title);
	
	gtk_notebook_set_tab_label_text (GTK_NOTEBOOK (widget), w, title);
	
	g_free (title);
}

/*! \brief Czyści zbiór

	Niszczy wszystkie plansze wchodzące w skład zbioru
*/
void LWBoardSet::clear ()
{
	for (GSList *l = list_board; l != NULL; l = l->next) {
		LWBoard *board = (LWBoard *) l->data;
		delete board;
	}

	g_slist_free (list_board);
	list_board = NULL;
}

/*! \brief Zwraca kontrolkę

	Zwraca kontrolkę zbioru
*/

GtkWidget *LWBoardSet::getWidget ()
{
	return widget;
}

/*! \brief Zwraca typ plansz

	Zwraca typ plansz przechowywanych w zbiorze
*/
LWType LWBoardSet::getType()
{
	return type;
}
