/* (C) Marcin Kwadrans <quar@vitea.pl> */

#include "include/support.h"
#include "include/piece.h"
#include "include/environment.h"

static GtkTargetEntry target_table[] = {
  { "application/lwizard", GTK_TARGET_SAME_APP, 0 }
};

static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);

static gboolean dndpermissions_table[5][5] = {
/* FROM / TO 	   ICONS, COMMANDS, WORLD, PROGRAM, PLAYGROUND */
/* ICONS      */ { FALSE,    FALSE,  TRUE,    TRUE,      FALSE },
/* COMMANDS   */ { FALSE,    FALSE, FALSE,    TRUE,      FALSE },
/* WORLD      */ {  TRUE,     TRUE,  TRUE,   FALSE,      FALSE },
/* PROGRAM    */ {  TRUE,     TRUE, FALSE,    TRUE,      FALSE },
/* PLAYGROUND */ { FALSE,    FALSE, FALSE,   FALSE,      FALSE }
};

static gboolean is_trash (LWPiece *piece)
{
		return ((LW_TYPE_COMMANDS == piece->row->board->getType()) ||
		(LW_TYPE_ICONS == piece->row->board->getType())) ? TRUE : FALSE;
}

/*! \brief Obsługa zdarzenia pobrania danych za pomocą mechanizmu DND
*/
static void  
source_drag_data_get  (GtkWidget          *widget,
						GdkDragContext    *context,
						GtkSelectionData  *selection_data,
						guint             info,
						guint             time,
						gpointer          data)
{
	/* Lista zmiennych nieużywanych */
	(void) widget;
	(void) context;
	(void) info;
	(void) time;

	gtk_selection_data_set (selection_data,
			    selection_data->target,
                            8, (guchar *) &data, sizeof (gpointer));  

}

/*! \brief Obsługa zdarzenia otrzymania danych za pomocą mechanizmu DND
*/
static void  
target_drag_data_received  (GtkWidget          *widget,
							GdkDragContext     *context,
							gint                x,
							gint                y,
							GtkSelectionData   *data,
							guint               info,
							guint               time,
							gpointer           userdata)

{
	/* Lista zmiennych nieużywanych */
	(void) widget;
	(void) x;
	(void) y;
	(void) info;
	
	if ((data->length == 0) || (data->format != 8)) {
    	gtk_drag_finish (context, FALSE, FALSE, time);
 
    	return;
    }
	
	LWPiece *sourcepiece = * ((LWPiece **) data->data);
	LWPiece *destpiece = (LWPiece *) userdata;

	/* Nie ruszać dodatkowych klocków na końcach wierszy,
	 * w trybie programowania */
	if (TRUE == sourcepiece->row->isPieceDummy(sourcepiece)) {
			gtk_drag_finish (context, TRUE, FALSE, time);
			return;
	}

	/* Przenoszenie jedynego klocka w lini do tej samej lini
	 * nie ma sensu, dodatkowo powoduje naruszenie ochrony pamięci
	 * w niektórych sytuacjach, niedopuść do takiej sytuacji */
	if ((sourcepiece->row == destpiece->row) &&
		(NULL == sourcepiece->row->getPieceNth(1))) {
			gtk_drag_finish (context, TRUE, FALSE, time);
			return;
	}

	if (dndpermissions_table [sourcepiece->row->board->getType()]
		[destpiece->row->board->getType()] == FALSE) {
			gtk_drag_finish (context, TRUE, FALSE, time);
			return;
	}
		
	/* Jeśli wynegocjowano akcję kopiowania */	
	if (context->action == GDK_ACTION_COPY) {
		if (TRUE == is_trash (destpiece))
			sourcepiece->clear();
		else
		if (LW_TYPE_PROGRAM == destpiece->row->board->getType()) {
			LWPiece *newpiece = new LWPiece (sourcepiece, destpiece->row);
			destpiece->row->insertPieceBefore (newpiece, destpiece);
		} else
			destpiece->copy (sourcepiece);
	}
	
	/* Jeśli wynegocjowano akcję przenoszonia */
	if (context->action == GDK_ACTION_MOVE) {
		if (TRUE == is_trash (destpiece))
			sourcepiece->row->removePiece (sourcepiece);
		else {
			LWPiece *newpiece = new LWPiece (sourcepiece, destpiece->row);
			destpiece->row->insertPieceBefore (newpiece, destpiece);
			sourcepiece->row->removePiece (sourcepiece);
		}
	}

	gtk_drag_finish (context, TRUE, FALSE, time);
}

/*! \brief Komponuj obrazek

	Komponuje obrazek z dwóch warstw
	\param image Kontrolka obrazka, w którym znajdzie się kompozycja
	\param forepixmap Obrazek na wierzchu
	\param backpixmap Obrazek pod spodem
*/
static void
composite_image(GtkWidget *image, LWPixmap *forepixmap, LWPixmap *backpixmap)
{
	g_return_if_fail (image != NULL);
	g_return_if_fail (forepixmap != NULL);
	g_return_if_fail (backpixmap != NULL);

	GdkPixbuf *tmppixbuf = gdk_pixbuf_copy (backpixmap->getPixbuf());

	gdk_pixbuf_composite (forepixmap->getPixbuf(), tmppixbuf, 
						0, 0, /* Współrzędne lewego górnego rogu */
						ICON_XSIZE, ICON_YSIZE, /* Wielkość skalowanego obrazka */
						0.0, 0.0, /* offset, nie przycinaj lewej strony i góry obrazka */
						1.0, 1.0, /* skala, nie skaluj obrazka */
						GDK_INTERP_NEAREST, /* interpolacja, najszybsza */
						255); /* Alfa dla obrazka pod spodem */

	gtk_image_set_from_pixbuf (GTK_IMAGE (image), tmppixbuf);
	g_object_unref (G_OBJECT (tmppixbuf));
}

/*! \brief Wskazówka

	Wskazówka skojarzona z klockiem
*/
GtkTooltips *LWPiece::tooltips=NULL;

void LWPiece::init (LWRow *row)
{
	widget = gtk_event_box_new();
	gtk_widget_set_size_request (widget, ICON_XSIZE, ICON_YSIZE);
	gtk_widget_show (widget);
	attach(row);
	
	/* Tworzy grupę wskazówek, jeśli jeszcze nie istneje,
	nie trzeba dbać o usuwanie grupy, ponieważ ze zniszczeniem
	ostatniej kontrolki która korzysta z grupy, sama grupa
	ulegnie zniszczeniu */
		
	if (tooltips == NULL)
		tooltips = gtk_tooltips_new ();
}

/*! \brief Konstruktor

	Tworzy klocek
	\param row Klocek będzie przynależał do wiersza
*/
LWPiece::LWPiece (LWRow *row):
	backpixmap(NULL), forepixmap(NULL), command(NULL), image(NULL), inverted_grounds(FALSE)
{
	init (row);	
}

/*! \brief Konstruktor

	Tworzy klocek
	\param row Klocek będzie przynależał do wiersza
	\param pixmap Klocek będzie zawierał ikonę 
*/
LWPiece::LWPiece (LWRow *row, LWPixmap *pixmap): 
	backpixmap(NULL), forepixmap(NULL), command(NULL), image(NULL), inverted_grounds(FALSE)
{
	init (row);
	
	setBackgroundPixmap (pixmap);
}

/*! \brief Konstruktor kopiujący

	Kopiuje klocek
	\param piece Kopiowany klocek
	\param pixmap Klocek będzie zawierał ikonę 
*/
LWPiece::LWPiece (const LWPiece *piece, LWRow *row): 
	backpixmap(NULL), forepixmap(NULL), command(NULL), image(NULL), inverted_grounds(FALSE)
{
	init (row);
	
	copy ((LWPiece *) piece);
}

/*! \brief Destruktor

	Niszczy klocek
*/
LWPiece::~LWPiece ()
{
	clear ();
	
	gtk_widget_destroy (widget);
}

/*! \brief Odtwarza klocek z węzła XML

	Odtwarza wygląd klocka na podstawie węzła drzewa XML
	\param node Węzeł na podstawie zostanie dokonana rekonstrukcja
*/
void LWPiece::restoreFromXML (xmlNode *node)
{
	g_return_if_fail (LWEnvironment::getPixmapSet() != NULL);
	g_return_if_fail (!xmlStrcasecmp (node->name, (xmlChar *) "Piece"));
	
	clear();
	
	gchar *name = (gchar *) xmlGetProp (node, (xmlChar *) "pixmap");

	if (name != NULL)
		setBackgroundPixmap (LWEnvironment::getPixmapSet()->getPixmap (name));

	gchar *cmd = (gchar *) xmlGetProp (node, (xmlChar *) "cmd");

	if (cmd != NULL)
		setCommand (cmd);
}

/*! \brief Zapisanie informacji o klocku w postaci węzła

	Zapisuje wygląd klocka w węźle drzewa XML. Węzeł będzie dzieckiem 
	rodzica który zostaje przekazany do metody.
	\param parent_node Węzeł rodzica
*/
void LWPiece::storeToXML (xmlNode *parent_node)
{
	g_return_if_fail (!xmlStrcasecmp (parent_node->name, (xmlChar *) "Row"));
	
	xmlNode *node = xmlNewChild(parent_node, NULL, (xmlChar *) "Piece", NULL);

	if (backpixmap != NULL)
		xmlNewProp (node, (xmlChar *) "pixmap", (xmlChar *) backpixmap->getName());

	if (command != NULL)
		xmlNewProp (node, (xmlChar *) "cmd", (xmlChar *) command->getName());
}

/*! \brief Pobranie polecenia

	Pobiera polecenie skojarzone z klockiem
	\return Pobrane polecenie
*/
LWCommand *LWPiece::getCommand ()
{
	return command;
}

/*! \brief Ustawienie polecenia

	Ustawia polecenie, które będzie skojarzone z klockiem
	\param cmd Polecenie
*/
void LWPiece::setCommand (gchar *cmd)
{
	command = LWCommand::factory (cmd);	
	
	g_return_if_fail (command != NULL);
	
	gtk_tooltips_set_tip (tooltips, widget, 
			command->getHint(), command->getHint());
}

/*! \brief Pobranie ikony pod klockiem

	Pobiera ikonę, która znajduje się pod klockiem
	\return Pobrana ikona
*/
LWPixmap *LWPiece::getBackgroundPixmap ()
{
	return backpixmap;
}

/*! \brief Ustawia ikonę na klocku

	Ustawia ikonę, która znajdzie się na klocku
	\param pixmap Ikona do ustawienia
*/
void LWPiece::setForegroundPixmap (LWPixmap *pixmap)
{
	/* Nic */
	if (backpixmap == NULL && pixmap == NULL)
		clear();
	else

	/* Tylko backpixmap */
	if (backpixmap != NULL && pixmap == NULL)
		gtk_image_set_from_pixbuf (GTK_IMAGE(image), backpixmap->getPixbuf());
	else

	/* Tylko forepixmap */
	if (backpixmap == NULL && pixmap != NULL) {
		clear();
		image = gtk_image_new_from_pixbuf (pixmap->getPixbuf());
		gtk_container_add (GTK_CONTAINER (widget), image);
		gtk_widget_show (image);
	}
	
	else
	
	if (inverted_grounds == FALSE)
		composite_image (image, pixmap, backpixmap);
	else
		composite_image (image, backpixmap, pixmap);
	
   	forepixmap = pixmap;
}

/*! \brief Ustawia ikonę pod klockiem

	Ustawia ikonę, która znajdzie się pod klockiem
	\param pixmap Ikona do ustawienia
*/
void LWPiece::setBackgroundPixmap (LWPixmap *pixmap)
{
	
	if (forepixmap == NULL)  /* Jeśli istnieje wyłącznie obrazek na klocku*/
		clear();
	
	if (pixmap == NULL)  /* Jeśli nie ma pixmapy pozostaw klocek bez obrazka pod klockiem */
		return;
	
	if (image == NULL) {      /* Jeśli klocek nie ma obrazka */
		image = gtk_image_new_from_pixbuf (pixmap->getPixbuf());
		gtk_container_add (GTK_CONTAINER (widget), image);
		gtk_widget_show (image);
	} else

	if (inverted_grounds == FALSE)
		composite_image (image, forepixmap, pixmap);
	else
		composite_image (image, pixmap, forepixmap);

   	backpixmap = pixmap;

	if (LW_TYPE_PLAYGROUND != row->board->getType())
		gtk_drag_source_set_icon_pixbuf (widget, pixmap->getPixbuf());
}

/*! \brief Włącza odwrócenie warstw

	Włącza lub wyłącza odwrócone interpretowanie warstw. To co
	było na klocku znajdzie się pod nim i vice versa
	\param inverted Jeśli prawda włączone zostanie odwrotne
	interpretowane warstw, jeśli fałsz wyłączone
*/
void LWPiece::enableInvertedGrounds (gboolean inverted)
{
	inverted_grounds = inverted;
	
	if (backpixmap != NULL && forepixmap != NULL)
		if (inverted_grounds == FALSE)
			composite_image (image, forepixmap, backpixmap);
		else
			composite_image (image, backpixmap, forepixmap);
		
}

/*! \brief Wyczyszczenie zawartości klocka

	Czyści zawartość klocka
*/
void LWPiece::clear ()
{
	forepixmap = NULL;
	backpixmap = NULL;
	command = NULL;
	
	if (image != NULL) {
		gtk_widget_destroy (image);
		image = NULL;
	}

	if (LW_TYPE_PLAYGROUND != row->board->getType())
		gtk_drag_source_set_icon_stock (widget, GTK_STOCK_DND);
}

/*! \brief Dostowuje zawartość klocka

	Dostosowuje zawartość klockado wygladu klocka zawartego w piece
	\param piece Klocek na podstawie którego nastąpi dostosowanie wyglądu
*/
void LWPiece::copy (LWPiece *piece)
{
	g_return_if_fail (piece != NULL);
	
	setForegroundPixmap(piece->forepixmap);
	setBackgroundPixmap(piece->backpixmap);
	
	if (piece->command != NULL) {
		command = piece->command->clone();
	
		gtk_tooltips_set_tip (tooltips, widget, 
								command->getHint(), command->getHint());
		
	}
	
}	

/*! \brief Kojarzenie klocka z wierszem

	Kojarzy klocek z wierszem, inicjacja drag'n'drop (metoda prywatna)
	\param row Wiersz z którym klocek zostanie skojarzony
*/
void LWPiece::attach (LWRow *row)
{
gint source_actions=0;
gint dest_actions=0;
	
	g_return_if_fail (row != NULL);
	g_return_if_fail (row->board != NULL);
	
	switch (row->board->getType ()) {
		case LW_TYPE_ICONS: 
			source_actions = GDK_ACTION_COPY;
			dest_actions = GDK_ACTION_COPY | GDK_ACTION_MOVE;
			break;
		
		case LW_TYPE_COMMANDS:
			source_actions = GDK_ACTION_COPY;
			dest_actions = GDK_ACTION_COPY | GDK_ACTION_MOVE; 
			break;
		
		case LW_TYPE_WORLD:
			source_actions = GDK_ACTION_COPY;
			dest_actions = GDK_ACTION_COPY;
			break;
		
		case LW_TYPE_PROGRAM:
			source_actions = GDK_ACTION_MOVE;
			dest_actions = GDK_ACTION_COPY | GDK_ACTION_MOVE; 
			break;
		
		case LW_TYPE_PLAYGROUND: 
			source_actions = 0;
			dest_actions = 0;
			break;
	}

    if (dest_actions != 0) {
 		gtk_drag_dest_set (widget,
		    GTK_DEST_DEFAULT_ALL,
		   	target_table, n_targets,
		   	(GdkDragAction) dest_actions);

    	g_signal_connect ( G_OBJECT(widget), "drag_data_received",
	 		G_CALLBACK(target_drag_data_received), (gpointer) this);
	}

	if (source_actions != 0) {
		gtk_drag_source_set (widget, GDK_BUTTON1_MASK,
        	target_table, n_targets, 
	    	(GdkDragAction) source_actions);

		g_signal_connect (G_OBJECT (widget), "drag_data_get",
			G_CALLBACK(source_drag_data_get), (gpointer) this);
	
	}
	
	this->row = row;
}

/*! \brief Pobranie kontrolki

	Pobiera kontrolkę
	\return Pobrana kontrolka
*/
GtkWidget *LWPiece::getWidget ()
{
	return widget;
}
